#===================================================
#
# ON THE VALUE OF VIRTUAL CURRENCIES
# WILKO BOLT AND MAARTEN R.C. VAN OORDT
#
# JOURNAL OF MONEY, CREDIT AND BANKING (FORTHCOMING)
# 
# Purpose	Script to Replicate Figure 2 in the Paper
# Language:	R version 3.1.2 (The R Foundation for Statistical Computing)
# Authors:	Wilko Bolt and Maarten R.C. van Oordt
# Date: 	7 December 2018
#
#===================================================



#===================================================
#
# SETUP
#
#===================================================
# Set working directory here:
# setwd("C:/yourpath")
library(lubridate)

last = function(x) {
	tail(x,n=1)
}
#===================================================



#===================================================
#
# READ DATA
#
#===================================================
# Read bitcoin price data from csv-file (source: Blockchain.info)
coinprice=read.csv(file="./Data/blockchain.csv",sep=",",header = TRUE)
coinprice[,1]=as.Date(coinprice[,1],format="%d/%m/%Y")

# Read passive bitcoins data from csv-file
coinactivity=read.csv(file="./Data/coinmetrics.csv",sep=",")
coinactivity[,"Date"]=as.Date(coinactivity[,1],format="%d/%m/%Y")
#===================================================



#===================================================
#
# CALCULATE PRICE WITHOUT SPECULATION
#
#===================================================
activecoinsm6=rep(NA,length(coinprice[,"Date"]))
totalcoins=activecoinsm6
totalcoinsm6=activecoinsm6
plot(coinprice,type="l")
mydata=coinprice
for (i in 1:length(coinprice$Date) ) {
	ia = last( which(coinactivity[,"Date"]==(coinprice$Date[i] %m+% months(6)) ) )
	im = last( which(coinactivity[,"Date"]==(coinprice$Date[i]) ) )
	activecoinsm6[i] = coinactivity[ia,"Active"]
	totalcoins[i] = coinactivity[im,"M"]
	totalcoinsm6[i] = coinactivity[ia,"M"]
}
mydata$Activem6  = activecoinsm6
mydata$Total   = totalcoins
mydata$Totalm6   = totalcoinsm6
mydata$Passive = mydata$Totalm6-mydata$Activem6
mydata$Raw_price_no_Speculation=((mydata$S*mydata$Total)-(mydata$S*mydata$Passive)) / mydata$Total

# SMOOTHING WITH EXPONENTIONAL WEIGHTED AVERAGE
availableprice=is.na(mydata$Raw_price_no_Speculation)==FALSE
lambda=0.85^(4/365.25)
mydata$Smooth_price_no_SpeculationEWA=NA
whichavailableprice= (which(availableprice==TRUE))
mydata$Smooth_price_no_SpeculationEWA[whichavailableprice[1]]=mydata$Raw_price_no_Speculation[whichavailableprice[1]]
for (i in 2:length(whichavailableprice)) {
	mydata$Smooth_price_no_SpeculationEWA[whichavailableprice[i]]=lambda*mydata$Smooth_price_no_SpeculationEWA[whichavailableprice[i-1]]+(1-lambda)*mydata$Raw_price_no_Speculation[whichavailableprice[i]]
}
#===================================================




#===================================================
#
# GENERATE FIGURE 2 IN THE PAPER
#
#===================================================
postscript("figure2.eps", horizontal=FALSE, width=8, height=5, onefile=FALSE, pointsize=12)
par(cex=1,mex=0.8)
plot.new()
sample = year(mydata$Date)>2010 & mydata$Date<=mydata$Date[last(whichavailableprice)]
plot(mydata$Date[sample],mydata$Raw_price_no_Speculation[sample],type="l",col="darkgrey",lwd=2,ylim=c(0,max(mydata$S[sample])),xlab="",ylab="")
points(mydata$Date[sample],mydata$S[sample],type="l",lwd=2)
points(mydata$Date[sample],mydata$Smooth_price_no_SpeculationEWA[sample],type="l",lwd=1,lty="longdash")	
dev.off()
#===================================================

